<?php
/* * ***********************************************************************
 * RedShop App for Watchful
 * Copyright (c) 2006-2014 Anything Digital & Inetis
 * ************************************************************************
 * Watchful.li is a web site monitoring and security service.
 * 
 * Although Watchful.li is a paid service, this program is free software;
 * you can redistribute it and/or modify it under the terms of the GNU 
 * General Public License as published by the Free Software Foundation; 
 * either version 2 of the License, or (at your option) any later version.
 *
 * This header must not be removed. Additional contributions/changes
 * may be added to this header as long as no information is deleted.
 * ************************************************************************
 * Get the latest version of this program at: https://watchful.li
 * *********************************************************************** */

defined('_JEXEC') or die();

//Require the watchfulliApps class in class.apps.php
require_once(JPATH_ADMINISTRATOR . '/components/com_watchfulli/classes/apps.php');
// We extend a base class to pass additional parameters in the language strings
// It will probably be included in the client but we also put it here so the app will work
// even for older version of the Watchful client.
class HikashopAppAlert extends AppAlert
{
    public $parameter1;
    public $parameter2;
    public $parameter3;

    public function HikashopAppAlert($level, $message, $parameter1 = null, $parameter2 = null, $parameter3 = null)
    {
        if ($level != null && $message != null)
        {
            $this->level = $level;
            $this->message = $message;
            $this->parameter1 = $parameter1;
            $this->parameter2 = $parameter2;
            $this->parameter3 = $parameter3;
        }
    }

}
class plgWatchfulliAppsHikashopApp extends watchfulliApps
{

    public function createAppAlert($level, $message, $parameter1 = null, $parameter2 = null, $parameter3 = null)
    {
        $alert = new HikashopAppAlert($level, $message, $parameter1, $parameter2, $parameter3);
        $this->addAlert($alert);
    }

    public function appMainProgram($oldValuesSerialized = null)
    {
        $this->setName("HikashopApp");
        $this->setDescription("Hikashop App for Watchful - This App checks if the sales average for the last period is changed since the previous period.");
        $db = & JFactory::getDbo();
        $debug = $this->params->get('debug', 0);

        // Component check
        if (!file_exists(JPATH_ADMINISTRATOR . "/components/com_hikashop/hikashop.php"))
        {
            $this->createAppAlert(1, "HIKASHOPAPP_NO_COMPONENT");
            return $this;
        }

        // Get sample average
        $sample = $this->params->get('sample', 365);
        $query = $this->buildQuery($sample);
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][HIKASHOPAPP] Sample query: $query");
        $db->setQuery($query);
        if ($order_total = $db->loadResult())
        {
            $avg_orders_sample = $order_total / $sample;
        }
        else
        {
            $this->createAppAlert(1, "HIKASHOPAPP_SAMPLE_DATA_ERROR");
            return $this;
        }
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][HIKASHOPAPP] Sample average: $avg_orders_sample");

        // Get period average
        $period = $this->params->get('period', 7);
        $query = $this->buildQuery($period);
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][HIKASHOPAPP] Period query: $query");
        $db->setQuery($query);
        if ($order_total = $db->loadResult())
        {
            $avg_orders_period = $order_total / $period;
        }
        else
        {
            $this->createAppAlert(1, "HIKASHOPAPP_PERIOD_DATA_ERROR");
            return $this;
        }
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][HIKASHOPAPP] Period average: $avg_orders_period");

        $threshold = $this->params->get('threshold', 20);
        if ($debug)
            $this->createAppAlert(1, "[DEBUG][HIKASHOPAPP] Variation threshold: $threshold");

        // Alert if variation is bigger than the threshold - decreased
        $variation = ($avg_orders_sample - $avg_orders_period) / $avg_orders_sample * 100;
        if ($variation > $this->params->get('threshold', 20))
        {
            $this->createAppAlert($this->params->get('errorlevel'), "HIKASHOPAPP_VARIATION_DECREASE", sprintf("%.0f", $threshold) . '%', $period, $sample);
        }

        // Alert if variation is bigger than the threshold - increased
        $variation = ($avg_orders_period - $avg_orders_sample) / $avg_orders_sample * 100;
        if ($variation > $this->params->get('threshold', 20))
        {
            $this->createAppAlert($this->params->get('errorlevel'), "HIKASHOPAPP_VARIATION_INCREASE", sprintf("%.0f", $threshold) . '%', $period, $sample);
        }

        return $this;
    }

   private function buildQuery( $interval )
   {
     $startdate = time() - ($interval*24*60*60);
     $status = $this->params->get('status',array());
     if(!is_array($status)) $status = array( $status );

     $query = "SELECT SUM(order_full_price) FROM #__hikashop_order
 WHERE order_created >= '$startdate'";

     // Add status filter
     if(!empty( $status ) ) {
       $query .= " AND order_status IN ('" . implode("','",$status) . "')";
     }

     return $query;
   }

}
